@@ -63,7 +63,10 @@ print.tdt <- function(x, digits=4, ...){
 			"by non-biological artefacts (e.g., sparseness of the data).", call.=FALSE)
 }
 
-tdt2way <- function(snp1, snp2, epistatic=TRUE,
+tdt2way <- function(...)
+	cat("tdt2way has been renamed to tdtGxG. So please use tdtGxG instead.")
+
+tdtGxG <- function(snp1, snp2, epistatic=TRUE,
 		model=c("additive", "dominant", "recessive"), add=FALSE,
 		warnError=TRUE){
 	require(survival)
@@ -233,7 +236,11 @@ colTDT <- function(mat.snp, model=c("additive", "dominant", "recessive"), fast=T
 	out
 }
 
-colTDT2way <- function(mat.snp, epistatic=TRUE, genes=NULL, maf=FALSE,
+colTDT2way <- function(...){
+	cat("colTDT2way has been renamed to colGxG. So please use colGxG instead.\n")
+}
+
+colGxG <- function(mat.snp, epistatic=TRUE, genes=NULL, maf=FALSE,
 		model=c("additive", "dominant", "recessive"), add=FALSE,
 		warnError=TRUE){
 	require(survival)
@@ -353,7 +360,7 @@ print.colTDT <- function(x, top=5, digits=4, ...){
 	pval <- format.pval(x$pval, digits=digits)
 	out <- data.frame(Coef=x$coef, OR=x$OR, Lower=x$lowerOR, Upper=x$upperOR, 
 		SE=x$se, Statistic=x$stat, "p-Value"=pval, Trios=x$usedTrios,
-		check.names=FALSE)
+		check.names=FALSE, stringsAsFactors=FALSE)
 	if(x$ia)
 		cat("      Genotypic TDT for Two-Way Interaction (Using 15 Pseudo Controls)",
 			"\n\n")
@@ -363,7 +370,7 @@ print.colTDT <- function(x, top=5, digits=4, ...){
 		"recessive"="Recessive"), "\n", 
 		if(x$add) "Model also contains the two respective individual SNPs.\n",
 		"\n", sep="")
-	if(length(x$coef) > top){
+	if(!is.na(top) && top>0 && top <= length(x$coef)){
 		ord <- order(x$pval)[1:top]
 		out <- out[ord,]
 		cat("Top", top, ifelse(x$ia, "SNP Interactions:\n", "SNPs:\n"))
@@ -521,223 +528,9 @@ print.colTDTepi <- function(x, top=5, digits=4, ...){
 	
 
 
-colTDTinter2way <- function(mat.snp1, mat.snp2, snp=NULL, epistatic=TRUE, maf=FALSE,
-		model=c("additive", "dominant", "recessive"), add=FALSE,
-		warnError=TRUE){
-	require(survival)
-	if(missing(mat.snp2)){
-		if(is.null(snp))
-			stop("Either mat.snp2 or snp has to be specified.")
-		mat.snp2 <- cbind(snp)
-	}
-	else{
-		if(!is.null(snp))
-			stop("If mat.snp2 is specified, snp cannot be specified.")
-	}
-	if(!is.matrix(mat.snp1))
-		stop("mat.snp1 has to be a matrix.")
-	if(!is.matrix(mat.snp2))
-		stop("mat.snp2 has to be a matrix.")
-	n.obs <- nrow(mat.snp1)
-	if(n.obs != nrow(mat.snp2))
-		stop("mat.snp1 must have the same number of rows as mat.snp2.")
-	if(n.obs %% 3 != 0)
-		stop("The matrices do not seem to contain trio data, as its number of rows is\n",
-			"not dividable by 3.")
-	if(is.null(rownames(mat.snp1)))
-		stop("mat.snp1 does not seem to be a matrix in genotype format,\n",
-			"as the names of the rows are missing.")
-	if(!is.null(rownames(mat.snp1)) && !is.null(rownames(mat.snp2)) && 
-		any(rownames(mat.snp1) != rownames(mat.snp2)))
-		stop("mat.snp1 must have the same row names as mat.snp2.")
-	if(any(!mat.snp1 %in% c(0, 1, 2, NA)))
-		stop("The values in mat.snp1 must be 0, 1, and 2.")
-	if(any(!mat.snp2 %in% c(0, 1, 2, NA)))
-		stop("The values in mat.snp2 must be 0, 1, and 2.")
-	type <- match.arg(model)
-	mat.code <- matrix(c(0,0,0,0, 0,0,1,1, 0,0,1,1, 1,0,0,1, 1,0,0,1, 1,1,1,1, 
-		1,1,1,1, 2,2,2,2, 1,1,2,2, 1,1,2,2, 2,1,1,2, 2,1,1,2, 0,1,1,2, 
-		1,0,1,2, 2,0,1,1, NA,NA,NA,NA), nrow=4)
-	cn <- c("000", "010", "100", "011", "101", "021", "201", "222", "121", "211",
-		"122", "212", "110", "111", "112", "NANANA")
-	colnames(mat.code) <- cn
-	n.snp1 <- ncol(mat.snp1)
-	mat.pseudo1 <- matrix(NA, 4/3 * n.obs, n.snp1)
-	for(i in 1:n.snp1){
-		mat.trio <- matrix(mat.snp1[,i], ncol=3, byrow=TRUE)
-		mat.trio[rowSums(is.na(mat.trio)) > 0, ] <- NA
-		code <- paste(mat.trio[,1], mat.trio[,2], mat.trio[,3], sep="")
-		if(any(!code %in% cn)){
-			tmp.ids <- !code %in% cn
-			warning(sum(tmp.ids), " trios in mat.snp1 show Mendelian errors. These are removed.",
-				call.=FALSE)
-			code[tmp.ids] <- "NANANA"
-		}
-		mat.pseudo1[,i] <- as.vector(mat.code[,code])
-	}
-	n.snp2 <- ncol(mat.snp2)
-	mat.pseudo2 <- matrix(NA, 4/3 * n.obs, n.snp2)
-	for(i in 1:n.snp2){
-		mat.trio <- matrix(mat.snp2[,i], ncol=3, byrow=TRUE)
-		mat.trio[rowSums(is.na(mat.trio)) > 0, ] <- NA
-		code <- paste(mat.trio[,1], mat.trio[,2], mat.trio[,3], sep="")
-		if(any(!code %in% cn)){
-			tmp.ids <- !code %in% cn
-			warning(sum(tmp.ids), " trios show Mendelian errors. These are removed.",
-				call.=FALSE)
-			code[tmp.ids] <- "NANANA"
-		}
-		mat.pseudo2[,i] <- as.vector(mat.code[,code])
-	}
-	if(maf){
-		mat.tmp <- mat.snp1[-seq(3, nrow(mat.snp1), 3), ]
-		mat.tmp <- mat.tmp[!duplicated(rownames(mat.tmp)),]
-		valMAF1 <- colSums(mat.tmp, na.rm=TRUE) / (2 * colSums(!is.na(mat.tmp)))
-		mat.tmp <- mat.snp2[-seq(3, nrow(mat.snp2), 3), ]
-		mat.tmp <- mat.tmp[!duplicated(rownames(mat.tmp)),]
-		valMAF2 <- colSums(mat.tmp, na.rm=TRUE) / (2 * colSums(!is.na(mat.tmp)))
-		valMAF <- list(mat.snp1=valMAF1, mat.snp2=valMAF2)
-	}
-	else
-		valMAF <- NULL
-	n.trio <- length(code)
-	if(epistatic)
-		return(colTDTinterEpi(mat.pseudo1, mat.pseudo2, n.trio, colnames(mat.snp1), colnames(mat.snp2),
-			valMAF=valMAF, warnError=warnError))
-	if(type=="dominant"){
-		mat.pseudo1 <- (mat.pseudo1 >= 1) * 1
-		mat.pseudo2 <- (mat.pseudo2 >= 1) * 1
-	}
-	if(type=="recessive"){
-		mat.pseudo1 <- (mat.pseudo1 > 1) * 1
-		mat.pseudo2 <- (mat.pseudo2 > 1) * 1
-	}
-	mat.ids <- cbind(rep.int(rep(1:4, e=4), n.trio), rep.int(1:4, 4*n.trio))
-	mat.ids <- mat.ids + 4 * rep(0:(n.trio-1), e=16)
-	y <- rep.int(c(1, rep.int(0, 15)), n.trio)
-	strat <- rep(1:n.trio, e=16)
-	coef <- se <- numeric(n.snp1 * n.snp2)
-	if(warnError){
-		wa <- options()$warn
-		on.exit(options(warn=wa))
-		options(warn=2)
-	}
-	for(i in 1:n.snp1){
-		x1 <- mat.pseudo1[mat.ids[,1], i]
-		for(j in 1:n.snp2){ 
-			x2 <- mat.pseudo2[mat.ids[,2], j]
-			IA <- x1 * x2
-			if(add)
-				c.out <- try(clogit(y ~ IA + x1 + x2 + strata(strat)), silent=TRUE)
-			else
-				c.out <- try(clogit(y ~ IA + strata(strat)), silent=TRUE)
-			k <- (i-1) * n.snp2 + j
-			coef[k] <- if(is(c.out, "try-error")) NA else c.out$coefficients[1]
-			se[k] <- if(is(c.out, "try-error")) NA else sqrt(diag(c.out$var))[1]
-		}
-	}
-	if(warnError)
-		options(warn=wa)
-	if(any(is.na(coef)))
-		warning("The fitting of some of the models has failed. A possible reason\n",
-			"is that the two corresponding SNPs might be in (strong) LD.\n",
-			"For these interactions, all statistics are therefore set to NA.", 
-			call.=FALSE)
-	stat <- (coef / se)^2
-	lower <- exp(coef - qnorm(0.975) * se)
-	upper <- exp(coef + qnorm(0.975) * se)
-	pval <- pchisq(stat, 1, lower.tail=FALSE)
-	if(any(pval <= .Machine$double.eps, na.rm=TRUE))
-		warning("Some of the p-values are very small (< 2.2e-16). These results might be\n",
-			"misleading, as the small p-values might be caused by non-biological artefacts\n",
-			"(e.g., sparseness of the data).", call.=FALSE)
-	rn1 <- if(is.null(colnames(mat.snp1))) paste("SNP", 1:n.snp1, sep="") else colnames(mat.snp1)
-	if(is.null(snp)){
-		rn2 <- if(is.null(colnames(mat.snp2))) paste("SNP", n.snp1 + (1:n.snp2), sep="") else colnames(mat.snp2)
-		names(coef) <- names(stat) <- names(pval) <- paste(rep(rn1, e=n.snp2), rep(rn2, n.snp1), sep=" : ")
-	}
-	else
-		names(coef) <- names(stat) <- names(pval) <- rn1
-	if(maf){
-		mat.maf <- cbind(rep(round(valMAF1, 4), e=n.snp2), rep(round(valMAF2, 4), n.snp1))
-		rownames(mat.maf) <- names(coef)
-		colnames(mat.maf) <- c("First SNP", "Second SNP")
-	}
-	else
-		mat.maf <- NULL
-	out <- list(coef=coef, se=se, stat=stat, pval=pval, OR=exp(coef), 
-		lowerOR=lower, upperOR=upper, ia=TRUE, type=type, add=add,
-		maf=valMAF, matMAF=mat.maf) 
-	class(out) <- "colTDT"
-	out	
-
-}
-	
-colTDTinterEpi <- function(mat.pseudo1, mat.pseudo2, n.trio, rn1, rn2, valMAF=NULL,
-		warnError=TRUE){
-	mat.x1 <- mat.pseudo1 - 1 
-	mat.z1 <- (mat.x1 == 0) - 0.5
-	mat.x2 <- mat.pseudo2 - 1
-	mat.z2 <- (mat.x2 == 0) - 0.5
-	mat.ids <- cbind(rep.int(rep(1:4, e=4), n.trio), rep.int(1:4, 4*n.trio))
-	mat.ids <- mat.ids + 4 * rep(0:(n.trio-1), e=16)
-	y <- rep.int(c(1, rep.int(0, 15)), n.trio)
-	strat <- rep(1:n.trio, e=16)
-	n.snp1 <- ncol(mat.pseudo1)
-	n.snp2 <- ncol(mat.pseudo2)
-	ll.main <- ll.full <- numeric(n.snp1 * n.snp2)
-	if(warnError){
-		wa <- options()$warn
-		on.exit(options(warn=wa))
-		options(warn=2)
-	}
-	for(i in 1:n.snp1){
-		x1 <- mat.x1[mat.ids[,1], i]
-		z1 <- mat.z1[mat.ids[,1], i]
-		for(j in 1:n.snp2){ 
-			x2 <- mat.x2[mat.ids[,2], j]
-			z2 <- mat.z2[mat.ids[,2], j]
-			woIA <- try(clogit(y ~ x1 + z1 + x2 + z2 + strata(strat)), silent=TRUE)
-			full <- try(clogit(y ~ x1 + z1 + x2 + z2 + x1*x2 + x1*z2 + z1*x2 + 
-				z1*z2 + strata(strat)), silent=TRUE)
-			k <- (i-1) * n.snp2 + j
-			ll.main[k] <- if(is(woIA, "try-error")) NA else woIA$loglik[2]
-			ll.full[k] <- if(is(full, "try-error")) NA else full$loglik[2]
-		}
-	}
-	if(warnError)
-		options(warn=wa)
-	if(any(is.na(ll.full)) | any(is.na(ll.main)))
-		warning("The fitting of some of the models has failed. A possible reason\n",
-			"is that the two respective SNPs might be in (strong) LD.\n",
-			"The corresponding test statistic and the p-value are thus set to NA.",
-			call.=FALSE)
-	stat <- -2 * (ll.main - ll.full)
-	pval <- pchisq(stat, 4, lower.tail=FALSE)
-	if(any(pval <= .Machine$double.eps, na.rm=TRUE))
-		warning("Some of the p-values are very small (< 2.2e-16). These results might be\n",
-			"misleading, as the small p-values might be caused by non-biological artefacts\n",
-			"(e.g., sparseness of the data).", call.=FALSE)
-	if(is.null(rn1))
-		rn1 <- paste("SNP", 1:n.snp1, sep="")
-	if(n.snp2 > 1){
-		if(is.null(rn2))
-			rn2 <- paste("SNP", n.snp1 + (1:n.snp2), sep="")
-		names(ll.full) <- names(stat) <- names(pval) <- paste(rep(rn1, e=n.snp2), rep(rn2, n.snp1), sep=" : ")
-	}
-	else
-		names(ll.full) <- names(stat) <- names(pval) <- rn1
-	if(!is.null(valMAF)){
-		mat.maf <- cbind(rep(round(valMAF[[1]], 4), e=n.snp2), rep(round(valMAF[[2]], 4), n.snp1))
-		rownames(mat.maf) <- names(ll.full)
-		colnames(mat.maf) <- c("First MAF", "Second MAF")
-	}
-	else
-		mat.maf <- NULL
-	out <- list(ll.main=ll.main, ll.full=ll.full, stat=stat, pval=pval,
-		maf=valMAF, matMAF=mat.maf)
-	class(out) <- "colTDTepi"
-	out
+colTDTinter2way <- function(...){
+	cat("colTDTinter2way has been removed from trio. Please use colGxG instead.\n",
+		"In colGxG, the argument genes can be used to specify the different sets of SNPs.\n", sep="")
 }
 	
 allBetweenCombs <- function(gene){
@@ -752,3 +545,37 @@ allBetweenCombs <- function(gene){
 	mat.comb
 }
 
+getMatPseudo <- function(mat.snp){
+	if(!is.matrix(mat.snp))
+		stop("mat.snp has to be a matrix.")
+	if(nrow(mat.snp) %% 3 != 0)
+		stop("mat.snp does not seem to contain trio data, as its number of rows is\n",
+			"not dividable by 3.")
+	if(is.null(rownames(mat.snp)))
+		stop("mat.snp does not seem to be a matrix in genotype format,\n",
+			"as the names of the rows are missing.")
+	if(any(!mat.snp %in% c(0, 1, 2, NA)))
+		stop("The values in mat.snp must be 0, 1, and 2.")
+	mat.code <- matrix(c(0,0,0,0, 0,0,1,1, 0,0,1,1, 1,0,0,1, 1,0,0,1, 1,1,1,1, 
+		1,1,1,1, 2,2,2,2, 1,1,2,2, 1,1,2,2, 2,1,1,2, 2,1,1,2, 0,1,1,2, 
+		1,0,1,2, 2,0,1,1, NA,NA,NA,NA), nrow=4)
+	cn <- c("000", "010", "100", "011", "101", "021", "201", "222", "121", "211",
+		"122", "212", "110", "111", "112", "NANANA")
+	colnames(mat.code) <- cn
+	n.snp <- ncol(mat.snp)
+	mat.pseudo <- matrix(NA, 4/3 * nrow(mat.snp), n.snp)
+	for(i in 1:n.snp){
+		mat.trio <- matrix(mat.snp[,i], ncol=3, byrow=TRUE)
+		mat.trio[rowSums(is.na(mat.trio)) > 0, ] <- NA
+		code <- paste(mat.trio[,1], mat.trio[,2], mat.trio[,3], sep="")
+		if(any(!code %in% cn)){
+			tmp.ids <- !code %in% cn
+			warning(sum(tmp.ids), " trios show Mendelian errors. These are removed.",
+				call.=FALSE)
+			code[tmp.ids] <- "NANANA"
+		}
+		mat.pseudo[,i] <- as.vector(mat.code[,code])
+	}
+	colnames(mat.pseudo)
+	mat.pseudo
+}
